"""
    Some functions and classes that are used internally by
    ZStyleSheets but are not part of the public API.
    These may well change - You have been warned.
"""
__version__= '4.0'

import string
import re

import ZSSRule
import ZSSAttribute

def translateSelector(selector):
    """
        Translate a selector into an ID
        
        Selectors can contain characters that are illegal in
        IDs so this converts them to something a little more
        Zope friendly.
        space   --> _s_
        +       --> _p_
        .       --> _d_
        #       --> _h_
        :       --> _c_
    """
    id=string.lower(string.join(selector))
    id=re.sub("\ ", "_s_", id)
    id=re.sub("\+", "_p_", id)
    id=re.sub("\.", "_d_", id)
    id=re.sub("\#", "_h_", id)
    id=re.sub("\:", "_c_", id)
    if id[0]=="_":
        id = "z" + id
    else:
        id=string.upper(id[0]) + id[1:]
    return id

def cookId(id, title, file):
    """
        Create an ID and title for an uploaded
        object that only has a filename.
        
        If the title is given, only create an ID.
        
        If the ID is given, don't change anything.
    """
    if not id and hasattr(file, 'filename'):
        filename=file.filename
        title=title or filename
        id=filename[max(string.rfind(filename, '/'),
                        string.rfind(filename, '\\'),
                        string.rfind(filename, ':'),
                        )+1:]                  
    return id, title

def cmp_list(List1, List2):
    """
        Compare the contents of two lists.
    """
    List1.sort()
    List2.sort()
    if len(List1)>len(List2):
        return -1
    if len(List1)<len(List2):
        return 1
    cmp_list_loop=0
    while cmp_list_loop<len(List1):
        v=cmp(List1[cmp_list_loop], List2[cmp_list_loop])
        if v!=0:
            return v
        cmp_list_loop=cmp_list_loop+1
    return 0

def cmp_dict(Dict1, Dict2):
    """
        Compare the contents of two dictionaries.
    """
    List1=Dict1.keys()
    List2=Dict2.keys()
    List1.sort()
    List2.sort()
    if len(List1)>len(List2):
        return -1
    if len(List1)<len(List2):
        return 1
    cmp_dict_loop=0
    while cmp_dict_loop<len(List1):
        v=cmp(List1[cmp_dict_loop], List2[cmp_dict_loop])
        if v!=0:
            return v
        v=cmp(Dict1[List1[cmp_dict_loop]], Dict2[List2[cmp_dict_loop]])
        if v!=0:
            return v
        cmp_dict_loop=cmp_dict_loop+1
    return 0

class _Rule:
    """
        A class that represents a 'halfway house' in the import
        of a flat CSS file. The structure is a tree of _Rule objects
        that is converted into ZSSRule and ZSSAttribute objects after
        processing.
        The processing takes the form of splitting the text lines
        of the flat CSS into ((selector ...), {'attr': 'value', ...})
        and then attempt to merge any matching rules (To eliminate
        redundancy and ease management)
    """
    def __init__(self, Selectors, Attribs, List):
        """
            Initialize this _Rule node
        """
        if type(Selectors)==type(''):
            Selectors=string.strip(Selectors)
            if string.find(Selectors, " ")!=0:
                Selectors=re.split(" ", Selectors)
        Attributes={}
        if type(Attribs)==type(''):
            for Attrib in re.split(";", Attribs):
                try:
                    (Id, Value)=re.split(":", Attrib)
                except:
                    pass
                else:
                    Id=string.strip(Id)
                    Value=string.strip(Value)
                    if Id:
                        Attributes[Id]=Value
        else:
            Attributes=Attribs
        self.Selector=[Selectors[0]]
        self.Attributes={}
        self.List=[]
        if len(Selectors)>1:
            self.List.append(_Rule(Selectors[1:], Attributes, self.List))
        else:
            self.Attributes=Attributes
        self._merge(List)

    def _create(self, Parent, Browsers):
        """
            Create ZSSRule and ZSSAttribute objects for this node and
            its children and add them to the Zope Tree.
        """
        if not hasattr(Parent, translateSelector(self.Selector)):
            NewObj=ZSSRule.ZSSRule('', '', selector=self.Selector, Browsers=Browsers)
            Parent._setObject(NewObj.id, NewObj)
        else:
            NewObj=getattr(Parent, translateSelector(self.Selector))
        for S in self.List:
            S._create(NewObj, Browsers)
        for K in self.Attributes.keys():
            if not hasattr(NewObj, K):
                NewObj._setObject(K, ZSSAttribute.ZSSAttr(K, self.Attributes[K], Browsers))

    def _dump(self, Context='', StopAt=-1):
        """
            Dump the Rule Tree from here down.
            This is mainly used for debugging the CSS interpreter
            but I will leave it in just in case anyone has another
            use for it.
        """
        strret = "%s%s : %s\n" % (Context, self.Selector, self.Attributes)
        if StopAt!=0:
            for S in self.List:
                strret = strret + S._dump(Context + "%s->" % self.Selector, StopAt-1)
        return strret

    def _merge(self, List):
        """
            Check to see if this node can be merged with any of its
            peers. Then recurse through the children.
        """
        if type(List)!=type([]):
            raise "Type Error, List should be of type 'list' not %s" % re.sub("<type|>", "", "%s" % type(List))
        _merge_loop_1=0
        while _merge_loop_1<len(List):
            if self!=List[_merge_loop_1]:
                if cmp_list(self.List, List[_merge_loop_1].List)==0 and cmp_dict(self.Attributes, List[_merge_loop_1].Attributes)==0:
                    for S in List[_merge_loop_1].Selector:
                        self.Selector.append(S)
                    del List[_merge_loop_1]
                    _merge_loop_1=_merge_loop_1-1
            _merge_loop_1=_merge_loop_1+1
        _merge_loop_1=0
        while _merge_loop_1<len(List):
            if self!=List[_merge_loop_1]:
                if cmp_list(self.Selector, List[_merge_loop_1].Selector)==0:
                    for S in List[_merge_loop_1].List:
                        self.List.append(S)
                    for A in List[_merge_loop_1].Attributes.keys():
                        self.Attributes[A]=List[_merge_loop_1].Attributes[A]
                    del List[_merge_loop_1]
                    _merge_loop_1=_merge_loop_1-1
                    _merge_loop_2=0
                    while _merge_loop_2<len(List):
                        List[_merge_loop_2]._merge(List)
                        _merge_loop_2=_merge_loop_2+1
            _merge_loop_1=_merge_loop_1+1
        _merge_loop_1=0
        while _merge_loop_1<len(self.List):
            self.List[_merge_loop_1]._merge(self.List)
            _merge_loop_1=_merge_loop_1+1

